/*
   GMB_MINE - a graphical Minesweeper clone by Gerard Monells

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GMB_MINE; if not, write to the Free Software Foundation
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
   (or visit the http://www.gnu.org/ or http://www.fsf.org/ homepage)
*/

/* Patch by Eric Auer 2007: Added Mouse driver, smaller Turbo C 2.01 */
/* Compile in tiny model (.com) with tcc -mt -lt -O -Z -w gmb_mine.c */
/* Original: www.programmersheaven.com/download/23881/download.aspx  */

#include <dos.h>;       /* Nedeed for int86() */
/* #include <stdio.h>; */    /* Needed for printf() */
#include <stdlib.h>;    /* Needed for rand() and srand() */
/* #include <time.h>;  */    /* Used time(0) for srand() */
#ifdef __TURBOC__
#include <string.h>;	/* strlen */
#include <io.h>;	/* write  */
#endif

#define ESC 27
#define R_BUTTON 9	/* tab */
#define L_BUTTON 32	/* space */
#define UP 72
#define DOWN 80
#define RIGHT 77
#define LEFT 75
#define THRESHOLD 4	/* minimum mouse movement that moves cursor */


int screen[40][30];     /* Number of mines between, nine if mine */
int protected[40][30];  /* 0 = hided, 1 = flag, 2 = enigma, 3 = shown */
int numbers[11][16][16] = {
            /* Zero */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* One */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* Two */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 7, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* Three */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 7, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* Four */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 7, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 7, 7, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* Five */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* Six */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* Seven */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 7, 7, 7, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 1, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* Eight */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 1, 7, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 1, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* Nine, means mine */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 0, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 0, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 0, 7, 0, 0, 0, 0, 0, 7, 0, 7, 7, 7, 8,
             7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 8,
             7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 8,
             7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 8,
             7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 8,
             7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 8,
             7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 8,
             7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 8,
             7, 7, 7, 0, 7, 0, 0, 0, 0, 0, 7, 0, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 0, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 0, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
            /* Ten, means explosioned mine */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
            12,12,12,12,12,12,12,12,12,12,12,12,12,12,12, 8,
            12,12,12,12,12,12,12, 0,12,12,12,12,12,12,12, 8,
            12,12,12,12,12,12,12, 0,12,12,12,12,12,12,12, 8,
            12,12,12, 0,12, 0, 0, 0, 0, 0,12, 0,12,12,12, 8,
            12,12,12,12, 0, 0, 0, 0, 0, 0, 0,12,12,12,12, 8,
            12,12,12, 0, 0, 0, 0, 0, 0, 0, 0, 0,12,12,12, 8,
            12,12,12, 0, 0, 0, 0, 0, 0, 0, 0, 0,12,12,12, 8,
            12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,12, 8,
            12,12,12, 0, 0, 0, 0, 0, 0, 0, 0, 0,12,12,12, 8,
            12,12,12, 0, 0, 0, 0, 0, 0, 0, 0, 0,12,12,12, 8,
            12,12,12,12, 0, 0, 0, 0, 0, 0, 0,12,12,12,12, 8,
            12,12,12, 0,12, 0, 0, 0, 0, 0,12, 0,12,12,12, 8,
            12,12,12,12,12,12,12, 0,12,12,12,12,12,12,12, 8,
            12,12,12,12,12,12,12, 0,12,12,12,12,12,12,12, 8,
            12,12,12,12,12,12,12,12,12,12,12,12,12,12,12, 8};
int control[3][16][16] = {
            /* Hided */
             8,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
             8, 8,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,15,
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
            /* Flag */
             8,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
             8, 8,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7,12,12,12,12,12,12, 7, 7, 7,15,15,
             8, 8, 7, 7, 7,12,12,12,12,12,12, 7, 7, 7,15,15,
             8, 8, 7, 7, 7,12,12,12,12,12,12, 7, 7, 7,15,15,
             8, 8, 7, 7, 7,12,12,12,12,12,12, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 0, 0, 0, 0, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,15,
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
            /* Enigma */
             8,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
             8, 8,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 0, 0, 0, 0, 0, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 7, 7,15,15,
             8, 8, 7, 7, 0, 0, 0, 7, 7, 7, 0, 0, 0, 7,15,15,
             8, 8, 7, 7, 0, 0, 7, 7, 7, 7, 7, 0, 0, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0, 0, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 0, 0, 0, 0, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 0, 0, 7, 7, 7, 7, 7,15,15,
             8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,15,15,
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,15,
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8};
int cursor[16][16] = {
            /* We'll make color 14 transparent */
             0, 0,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
             0,15, 0,14,14,14,14,14,14,14,14,14,14,14,14,14,
             0,15,15, 0,14,14,14,14,14,14,14,14,14,14,14,14,
             0,15,15,15, 0,14,14,14,14,14,14,14,14,14,14,14,
             0,15,15,15,15, 0,14,14,14,14,14,14,14,14,14,14,
             0,15,15,15,15,15, 0,14,14,14,14,14,14,14,14,14,
             0,15,15,15,15,15,15, 0,14,14,14,14,14,14,14,14,
             0,15,15,15,15,15,15,15, 0,14,14,14,14,14,14,14,
             0,15,15,15,15,15,15,15,15, 0,14,14,14,14,14,14,
             0,15,15,15,15,15, 0, 0, 0, 0, 0,14,14,14,14,14,
             0,15,15, 0,15,15, 0,14,14,14,14,14,14,14,14,14,
             0,15, 0,14, 0,15,15, 0,14,14,14,14,14,14,14,14,
             0, 0,14,14, 0,15,15, 0,14,14,14,14,14,14,14,14,
             0,14,14,14,14, 0,15,15, 0,14,14,14,14,14,14,14,
            14,14,14,14,14, 0,15,15, 0,14,14,14,14,14,14,14,
            14,14,14,14,14,14, 0, 0,14,14,14,14,14,14,14,14};
int not_mine[16][16] = {
            /* This position wasn't a mine; shown in show_all_mines() */
             8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 0, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7,12,12, 7, 7, 7, 0, 7, 7, 7,12,12, 7, 7, 8,
             7, 7,12,12,12, 0, 0, 0, 0, 0,12,12,12, 7, 7, 8,
             7, 7, 7,12,12,12, 0, 0, 0,12,12,12, 7, 7, 7, 8,
             7, 7, 7, 0,12,12,12, 0,12,12,12, 0, 7, 7, 7, 8,
             7, 7, 7, 0, 0,12,12,12,12,12, 0, 0, 7, 7, 7, 8,
             7, 0, 0, 0, 0, 0,12,12,12, 0, 0, 0, 0, 0, 7, 8,
             7, 7, 7, 0, 0,12,12,12,12,12, 0, 0, 7, 7, 7, 8,
             7, 7, 7, 0,12,12,12, 0,12,12,12, 0, 7, 7, 7, 8,
             7, 7, 7,12,12,12, 0, 0, 0,12,12,12, 7, 7, 7, 8,
             7, 7,12,12,12, 0, 0, 0, 0, 0,12,12,12, 7, 7, 8,
             7, 7,12,12, 7, 7, 7, 0, 7, 7, 7,12,12, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 0, 7, 7, 7, 7, 7, 7, 7, 8,
             7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8};
int has_mouse = -1; /* do mouse install check only once */

int get_mouse(void);
int read_mouse(int * x, int * y);
void myprint(char * t);
void myclrscr(void);
int mygetch(void);
int mykeywait(void);
void set_mode(int m);
int get_mode(void);
void put_pixel(int x, int y, int c);
void menu(void);
void show_options(void);
void game(int width, int heigth, int mines);
void prepare_screen(int width, int heigth, int mines);
void draw_screen(int width, int heigth, int off_x, int off_y);
void draw_position(int x, int y, int off_x, int off_y);
void show_cursor(int x, int y, int off_x, int off_y);
void hide_cursor(int x, int y, int off_x, int off_y, int width, int height);
void draw_black(int x, int y, int off_x, int off_y);
void draw_not_mine(int x, int y, int off_x, int off_y);
void show_all_mines(int width, int height, int off_x, int off_y);
void show_all_flags(int width, int height, int off_x, int off_y);
int show_between(int x, int y, int off_x, int off_y, int width, int height);


int get_mouse() {
  union REGS r;
  if (has_mouse==-1) {
    r.x.ax = 0; /* install check */
    int86(0x33, &r, &r);
    has_mouse = (r.x.ax == 0xffff) ? 1 : 0; /* 0xffff -> installed */
  }
  return (has_mouse == 1);
}

int read_mouse(int * x, int * y) {
  union REGS r;
  int btn = 0;
  r.x.ax = 3; /* read button states */
  r.x.bx = 0; /* ...nothing pressed */
  if (get_mouse()) do {
    btn = r.x.bx; /* "previous state" 1 left 2 right 4 middle */
    int86(0x33, &r, &r); /* bx buttons, cx column, dx row */
  } while (r.x.bx!=0); /* wait until all buttons released */
  r.x.ax = 0x0b; /* read movement since last call */
  r.x.cx = 0;
  r.x.dx = 0;
  if (get_mouse()) int86(0x33, &r, &r);
  *x = r.x.cx; /* mickeys moved since last call */
  *y = r.x.dx; /* ax: buttons bx: count cx/dx col/row of release */
  return btn;
}

void myprint(char * t) { /* printf() makes the binary big */
  write(1, t, strlen(t));
}

void myclrscr() { /* crt stuff makes the binary big */
  set_mode(3);
}

int mygetch(void) {
  union REGS r;
  r.x.ax = 0; /* fetch key */
  int86(0x16, &r, &r);
  return r.x.ax;
}

int mykeywait(void) {
  union REGS r;
  r.x.ax = 0x100; /* check for key */
  int86(0x16, &r, &r);
  return (r.x.flags & 0x40) == 0; /* ZF set if no key */
}

unsigned mytime(int n) { /* time() makes the binary big */
  return (unsigned)peek(0x40,0x6c) + n;
}


/* IN: nothing */
/* OUT: plays the game */
void main() {
  menu();
}

/* IN: a valid mode for screen */
/* OUT: mode m is set */
void set_mode(int m) {
  union REGS r;
  r.x.ax = m;
  int86(0x10, &r, &r);
}

/* IN: nothing */
/* OUT: return video mode */
int get_mode() {
  union REGS r;
  r.h.ah = 15;
  int86(0x10, &r, &r);
  return (r.h.al);
}

/* IN: valid coordinates and color for the video mode */
/* OUT: a pixel is set in x,y coordinates with color c */
void put_pixel(int x, int y, int c) {
  union REGS r;
  r.h.ah = 0x0C;
  r.h.al = c;
  r.h.bh = 0;
  r.x.cx = x;
  r.x.dx = y;
  int86(0x10, &r, &r);
}

/* IN: nothing */
/* OUT: the main menu */
void menu() {
  int option;
  while (1==1) {
    show_options();
    option = mygetch() & 0xff;
    switch (option) {
      case '1':
        game(8, 8, 10);
        break;
      case '2':
        game(16, 16, 40);
        break;
      case '3':
        game(30, 16, 99);
        break;
      default:
        return;
    } /* switch */
  } /* while */
}

/* IN: nothing */
/* OUT: shows menu options */
void show_options() {
  myclrscr();
  myprint("\nGMB_MINE v1.1 by Gerard Monells\n");
  myprint("gerardmonells -at- hotmail .com\n");
  myprint("Free open source  under the GNU\n");
  myprint("Public License v3, see gpl3.txt\n\n");
  myprint("-------------------------------\n\n");
  myprint("1     - easy game\n");
  myprint("2     - medium game\n");
  myprint("3     - hard game\n");
  myprint("esc   - exit\n\n");
  myprint("-------------------------------\n\n");
  myprint("space - clear\n");
  myprint("tab   - flag\n");
  myprint("arrow - move\n");
  myprint("esc   - exit\n\n");
}

/* IN: 0 <= width <= 40 and 0 <= height <= 30 and mines <= width * height */
/* OUT: nothing; game played */
void game(int width, int height, int mines) {
  /* Some Variables */
  int old_mode;
  int cur_x = 0, cur_y = 0, s;
  int option;
  int finished = 0;
  int remaining_positions = width * height;
  int off_x = ((40 - width) * 16) /2;
  int off_y = ((30 - height) * 16) /2;
  /* Prepare screen matrix and init graphics */
  prepare_screen(width, height, mines);
  old_mode = get_mode(); set_mode(0x12);
  /* Ready to begin; show graphics */
  draw_screen(width, height, off_x, off_y);
  show_cursor(cur_x, cur_y, off_x, off_y);
  /* The main loop */
  while (finished == 0) {

    option = 0;
    if (mykeywait()) {
      option = mygetch();
    } else {
      int x, y;
      switch (read_mouse(&x, &y)) {
        case 1: /* left */
          option = L_BUTTON;
          break;
        case 2: /* right */
          option = R_BUTTON;
          break;
        case 0: /* none */
          if (x < -THRESHOLD) {
            option = LEFT << 8;
          } else if (x > THRESHOLD) {
            option = RIGHT << 8;
          } else if (y < -THRESHOLD) {
            option = UP << 8;
          } else if (y > THRESHOLD) {
            option = DOWN << 8;
          } /* else no significant movement */
          break;
#if 0 /* need keyboard in menu and to leave completed game anyway */
        case 3: /* left and right */
        case 4: /* middle (if any) */
          option = ESC;
          break;
        default: /* other button combos */
          /* do nothing */
          break;
#endif
      } /* switch */
    } /* no key waiting */

    if ((option & 0xff) == 0) {
      /* use scancode, not ASCII */
      switch (option >> 8) {
      /* Hide cursor, change coors if not going out, and show cursor */
        case UP:
          if (cur_y != 0) {
            hide_cursor(cur_x, cur_y, off_x, off_y, width, height);
            cur_y--;
            show_cursor(cur_x, cur_y, off_x, off_y);
          }
          break;
        case DOWN:
          if (cur_y != height - 1) {
            hide_cursor(cur_x, cur_y, off_x, off_y, width, height);
            cur_y++;
            show_cursor(cur_x, cur_y, off_x, off_y);
          }
          break;
        case LEFT:
          if (cur_x != 0) {
            hide_cursor(cur_x, cur_y, off_x, off_y, width, height);
            cur_x--;
            show_cursor(cur_x, cur_y, off_x, off_y);
          }
          break;
        case RIGHT:
          if (cur_x != width - 1) {
            hide_cursor(cur_x, cur_y, off_x, off_y, width, height);
            cur_x++;
            show_cursor(cur_x, cur_y, off_x, off_y);
          }
          /* Another key, nothing is done */
      }
    } else {
      /* Not an arrow; what is? */
      switch (option & 0xff) {
        case L_BUTTON:
          /* Nothing is done if it's shown or with a flag */
          if (protected[cur_x][cur_y] == 0 || protected[cur_x][cur_y] == 2) {
            /* Other case, show position */
            protected[cur_x][cur_y] = 3;
            /* Don't want cursor; repaints new position */
            hide_cursor(cur_x, cur_y, off_x, off_y, width, height);
            /* There's not a mine? */
            if (screen[cur_x][cur_y] != 9) {
              /* Okay; not mine; if zero, show "coast of numbers" */
              if (screen[cur_x][cur_y] == 0) {
                s = show_between(cur_x, cur_y, off_x, off_y, width, height);
                remaining_positions = remaining_positions - s;
              }
              /* All numbers (zero too) does this */
              remaining_positions--;
              /* End: remaining_positions == mines will end */
              if (remaining_positions == mines) {
                /* If finished put all flags, wait for a key
                   and finish a message could be here */
                show_all_flags(width, height, off_x, off_y);
                mygetch();
                finished = 1;
              } else {
                /* If not finished, restore cursor; keep playing */
                show_cursor(cur_x, cur_y, off_x, off_y);
              }
            } else {
              /* Boom! Put an explosiones mine; draw it;
                 show all mines; except ones with flag */
              screen[cur_x][cur_y] = 10;
              draw_position(cur_x, cur_y, off_x, off_y);
              show_all_mines(width, height, off_x, off_y);
              /* Wait for a key; and finish; a message could be here */
              mygetch();
              finished = 1;
            }
          }
          break;
        case R_BUTTON:
          if (protected[cur_x][cur_y] != 3) {
            /* protected[cur_x][cur_y] increments; if == 3 then = 0 */
            protected[cur_x][cur_y] = (protected[cur_x][cur_y] + 1) % 3;
            draw_position(cur_x, cur_y, off_x, off_y);
            show_cursor(cur_x, cur_y, off_x, off_y);
          }
          break;
        case ESC:
          /* You ESCaped! A message could be here */
          finished = 1;
        /* Another key, nothing is done */
      }
    }
  }
  /* Finished; let's restore screen */
  set_mode(old_mode);
}

/* IN: 0 <= width <= 40 and 0 <= height <= 30 and mines <= width * height */
/* OUT: nothing; screen[][] and protected[][] are ready to play */
void prepare_screen(int width, int height, int mines) {
  int i, j, m = 0;
  int a, b;
  srand(mytime(0));
  /* Blank screen[][] */
  for (j = 0; j < height; j++) {
    for (i = 0; i < width; i++) {
      screen[i][j] = 0;
    }
  }
  /* Generate mines */
  while (mines != m) {
	i = rand() % width;
	j = rand() % height;
    if (screen[i][j] != 9) {
      screen[i][j] = 9;
      m++;
      /* Increment number of mines between */
      for (b = -1; b < 2; b++) {
        for (a = -1; a < 2; a++) {
          if (i+a < width && i+a >= 0 && j+b < height && j+b >= 0) {
            if (screen[i + a][j + b] != 9) { screen[i + a][j + b]++; }
          }
        }
      }
    }
  }
  /* Protection matrix; all is hided */
  for (j = 0; j < height; j++) {
    for (i = 0; i < width; i++) {
      protected[i][j] = 0;
    }
  }
}

/* IN: 0 <= width <= 40 and 0 <= height <= 30 */
/* OUT: nothing; redraw full screen */
void draw_screen(int width, int height, int off_x, int off_y) {
  int i, j;
#if 1 /* enable fancy screen update by Eric :-) */
  for (i = 0; i < width; i+=2) {
    for (j = 0; j < height; j+=2) {
      draw_position(i, j, off_x, off_y);
    }
    for (j = ((height & -2) - 1); j >= 0; j-=2) {
      draw_position(i, j, off_x, off_y);
    }
  }
  for (i = ((width & -2) - 1); i >= 0; i-=2) {
    for (j = 0; j < height; j+=2) {
      draw_position(i, j, off_x, off_y);
    }
    for (j = ((height & -2) - 1); j >= 0; j-=2) {
      draw_position(i, j, off_x, off_y);
    }
  }
#else
  for (j = 0; j < height; j++) {
    for (i = 0; i < width; i++) {
      draw_position(i, j, off_x, off_y);
    }
  }
#endif
}

/* IN: x between 0 and width; y between 0 and height */
/* OUT: draw in the screen what it is in this position */
void draw_position(int x, int y, int off_x, int off_y) {
  int i, j, c, a, b;
  a = 16 * x + off_x; b = 16 * y + off_y;
  c = protected[x][y];
  if (c == 3) {
    c = screen[x][y];
    for (j = 0; j < 16; j++) {
      for (i = 0; i < 16; i++) {
        put_pixel(a + i, b + j, numbers[c][j][i]);
      }
    }
  } else {
    for (j = 0; j < 16; j++) {
      for (i = 0; i < 16; i++) {
        put_pixel(a + i, b + j, control[c][j][i]);
      }
    }
  }
}

/* IN: valid x,y coordinates */
/* OUT: draws cursor in x,y coordinates */
void show_cursor(int x, int y, int off_x, int off_y) {
  int a = x * 16 + off_x + 7; int b = y * 16 + off_y + 7; int i, j, c;
  for (j = 0; j < 16; j++) {
    for (i = 0; i < 16; i++) {
      c = cursor[j][i];
      if (c != 14) { put_pixel(a + i, b + j, c); }
    }
  }
}

/* IN: valid x,y coordinates */
/* OUT: redraws position x,y and near, to hide cursor */
void hide_cursor(int x, int y, int off_x, int off_y, int width, int height) {
  draw_position(x, y, off_x, off_y);
  /* x+1 inside screen? */
  if (x < width - 1) {
    draw_position(x + 1, y, off_x, off_y);
  } else {
    draw_black(x + 1, y, off_x, off_y);
  }
  /* y+1 inside screen? */
  if (y < height - 1) {
    draw_position(x, y + 1, off_x, off_y);
  } else {
    draw_black(x, y + 1, off_x, off_y);
  }
  /* both inside screen? */
  if (x < width - 1 && y < height -1) {
    draw_position(x + 1, y + 1, off_x, off_y);
  } else {
    draw_black(x + 1, y + 1, off_x, off_y);
  }
}

/* IN: coordinates, offsets */
/* OUT: draws black square in x,y position */
void draw_black(int x, int y, int off_x, int off_y) {
  int i, j; int a = x * 16 + off_x; int b = y * 16 + off_y;
  for (j = 0; j < 16; j++) {
    for (i = 0; i < 16; i++) {
      put_pixel(a + i, b + j, 0);
    }
  }
}

/* IN: coordinates, offsets */
/* OUT: draws a not-mine position in x,y coordinates */
void draw_not_mine(int x, int y, int off_x, int off_y) {
  int i, j; int a = x * 16 + off_x; int b = y * 16 + off_y;
  for (j = 0; j < 16; j++) {
    for (i = 0; i < 16; i++) {
      put_pixel(a + i, b + j, not_mine[j][i]);
    }
  }
}

/* IN: nothing */
/* OUT: show all mines */
void show_all_mines(int width, int height, int off_x, int off_y) {
  int i, j;
  for (j = 0; j < height; j++) {
    for (i = 0; i < width; i++) {
      if (protected[i][j] != 1 && screen[i][j] == 9) {
        protected[i][j] = 3;
        draw_position(i, j, off_x, off_y);
      }
      if (protected[i][j] == 1 && screen[i][j] != 9) {
        draw_not_mine(i, j, off_x, off_y);
      }
    }
  }
}

/* IN: nothing */
/* OUT: puts flags on all hided mines */
void show_all_flags(int width, int height, int off_x, int off_y) {
  int i, j;
  for (j = 0; j < height; j++) {
    for (i = 0; i < width; i++) {
      if (protected[i][j] == 0 || protected[i][j] == 2) {
        protected[i][j] = 1;
        draw_position(i, j, off_x, off_y);
      }
    }
  }
}

/* IN: valid parameters */
/* OUT: show all hided positions between given, and central one,
        recursing if zero; returns positions shown */
int show_between(int x, int y, int off_x, int off_y, int width, int height) {
  int i, j, s = 0;
  for (j = -1; j < 2; j++) {
    for (i = -1; i < 2; i++) {
      /* Inside screen? */
      if (x+i>=0 && x+i<width && y+j>=0 && y+j<height) {
        /* Not shown? */
        if (protected[x + i][y + j] != 3) {
          /* Let's show it */
          protected[x + i][y + j] = 3;
          draw_position(x + i, y + j, off_x, off_y);
          /* One more shown! */
          s++;
          /* If zero, recurse */
          if (screen[x + i][y + j] == 0) {
            s = s + show_between(x + i, y + j, off_x, off_y, width, height);
          }
        }
      }
    }
  }
  return(s);
}
